#pragma once

#include "..\core\common.h"
#include "..\core\WMPacket.h"
#include "..\core\globals.h"

class HField;

struct rgb48_data_element {
public:
	rgb48_data_element() { };
	rgb48_data_element(float _r, float _g, float _b) { r = _r; g = _g; b = _b; };
	void CreateFrom(DWORD rgb);
	DWORD GetPacked();

	rgb48_data_element operator+( rgb48_data_element &second );
	rgb48_data_element operator-( rgb48_data_element &second );
	rgb48_data_element operator*( rgb48_data_element vector );
	rgb48_data_element operator*( float scalar );

	float r,g,b;
};

inline rgb48_data_element operator*( float scalar, rgb48_data_element &value ) {
	rgb48_data_element result;
	result.r = value.r * scalar;
	result.g = value.g * scalar;
	result.b = value.b * scalar;
	return result;
};

inline rgb48_data_element rgb48_data_element::operator+( rgb48_data_element &other ) {
	rgb48_data_element result;
	result.r = r + other.r;
	result.g = g + other.g;
	result.b = b + other.b;
	return result;
};

inline rgb48_data_element rgb48_data_element::operator-( rgb48_data_element &other ) {
	rgb48_data_element result;
	result.r = r - other.r;
	result.g = g - other.g;
	result.b = b - other.b;
	return result;
};

inline rgb48_data_element rgb48_data_element::operator*( float scalar ) {
	rgb48_data_element result;
	result.r = r * scalar;
	result.g = g * scalar;
	result.b = b * scalar;
	return result;
};

inline rgb48_data_element rgb48_data_element::operator*( rgb48_data_element vect ) {
	rgb48_data_element result;
	result.r = r * vect.r;
	result.g = g * vect.g;
	result.b = b * vect.b;
	return result;
};

typedef rgb48_data_element colorF;

class BMPacket :
	public WMPacket
{
public:
	BMPacket();									// Protect constructor/destructor
	BMPacket(const BMPacket &other);			// copy construction
	BMPacket &operator=(const BMPacket &other);

	virtual int getType() { return WMP_rgb; };
	virtual char *getTypeName() { return "Bitmap"; };
	virtual dword Color() { return 0x00ff0000; };

	virtual WMPacket *Clone();			// Return a copy of this WMPacket-derived packet
	virtual bool Load(std::istream &input);	// Load from stream input
	virtual bool Save(std::ostream &output);	// Save a copy of this data to stream output

// initializes a BMP's data store to the size specified by the SizeData structure
// after initialization the bmp is ready for read/write operations
	void Init(SizeData _size);
	void InitFrom(BMPacket &other);		// Match other's attributes, but DOES NOT copy data contents! Use copy constructor if you want that
	virtual void Clear();

	void ClampRange();				// Clamp data to lie 0..1.

	bool CreateFromHeightField(HField *other);
	SizeData getSizeData() { return sd; };
	void	 setSizeData(SizeData &_sd) { sd = _sd; };

	virtual int getHeapUsage() { if (!data) return 0; return sd.size.x * sd.size.y * sizeof(colorF);	};

// Heightfield size accessor functions
	inline int w() const { return sd.size.x; };
	inline int h() const { return sd.size.y; };
	inline int area() const { return sd.size.x * sd.size.y; };

	rgb48_data_element *GetDataPtr();			// Return a pointer to heightfield memory. USE WITH CAUTION!!!
	inline rgb48_data_element &operator[](const int index);	// low overhead
	inline rgb48_data_element value(const Coord loc);		// fastest coordinate-based accessor
	inline rgb48_data_element &operator[](const Coord loc);

	rgb48_data_element valueSafe(const Coord loc);	// Clips coordinates so that out-of-range access works (slower)
	rgb48_data_element valueSafeF(const CoordF loc); // Bilinear sampled data
protected:
	virtual void Kill();		// Call to delete its own type

	virtual void clearContents();
	virtual bool isTypePageable() { return true; };


// data array
	rgb48_data_element *data;		// pointer to data
	SizeData sd;

	virtual ~BMPacket();
};

// Safe Accessor inline function to provide a safe way to cast WMPacket to HField
// in debug mode it will assert if the packet is not a HF
inline BMPacket *BMP(WMPacket *pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_rgb;
		if (!ok)
			return NULL;
	}
	return (BMPacket*) pack; 
};

inline BMPacket *GetEmptyBMP() {
	BMPacket *bmp = BMP(WMGlobal->GetPacket(WMP_rgb));
	return bmp; 
};

inline BMPacket *GetNewBMP(SizeData sd) {
	BMPacket *bmp = BMP(WMGlobal->GetPacket(WMP_rgb));
	bmp->Init(sd);
	return bmp; 
};


// The inline data accessors
inline rgb48_data_element BMPacket::value(const Coord loc) {
	return data[((int)loc.y*sd.size.x) + (int)loc.x];
};

inline rgb48_data_element &BMPacket::operator[](const int index) {
	return data[index];
}

inline rgb48_data_element &BMPacket::operator[](const Coord loc) {
	return data[(loc.y*sd.size.x) + loc.x];
}

inline rgb48_data_element *BMPacket::GetDataPtr() {
	return data;
}
