#ifndef _GEOMETRY_H
#define _GEOMETRY_H
#pragma once

#include <math.h>
#include <vector>
#include "common.h"

class Geometry {
};

// 2D
class Vector2D {
public:
	Vector2D() {};
	Vector2D(double nx, double ny) { x = nx; y = ny; };
	double x,y;
	inline void operator*=( const double scalar ) {
		x *= scalar;
		y *= scalar;
	};
};

class Vector2 {
public:
	Vector2() {};
	Vector2(float nx, float ny) { x = nx; y = ny; };
//	Vector2(Coord &a) { x = (float) a.x; y = (float) a.y; };
	friend Vector2 operator+( const Vector2 &first, const Vector2 &second );
	friend Vector2 operator-( const Vector2 &first, const Vector2 &second );
	friend Vector2 operator*( const Vector2 &first, const float scalar );
	friend Vector2 operator*( const Vector2 &first, const Vector2 &second );
	friend Vector2 operator/( const Vector2 &first, const float scalar );
	friend Vector2 operator/( const Vector2 &first, const Vector2 &second );
	friend float dot( const Vector2 &first, const Vector2 &second );
	inline void operator+=( const Vector2 &first) {
		x += first.x;
		y += first.y;
	};
	inline void operator-=( const Vector2 &first) {
		x -= first.x;
		y -= first.y;
	};
	inline void operator/=( const float divisor ) {
		x /= divisor;
		y /= divisor;
	};
	inline void operator/=( const Vector2 &first ) {
		x /= first.x;
		y /= first.y;
	};
	inline void operator*=( const float scalar ) {
		x *= scalar;
		y *= scalar;
	};
	inline void operator*=( const Vector2 &second ) {
		x *= second.x;
		y *= second.y;
	};

	inline float length() const  { return (float) sqrt((x*x)+(y*y)); };
	inline float length2() const  { return ((x*x)+(y*y)); };			// length squared
//	inline float lengthManhattan() const  { return (float) (x*x)+(y*y); };


	inline void normalize() { float size = length(); if (size > 0) { x /= size; y /= size; }};
	inline Vector2 direction() {	// Unit vector in direction of this vector
		float size = length();
		if (size)
			return Vector2(x / size, y / size);
		else
			return Vector2(0,0);
	};

	float x,y;
};

inline Vector2 operator+( const Vector2 &first, const Vector2 &second ) {
	Vector2 result;
	result.x = first.x + second.x;
	result.y = first.y + second.y;
	return result;
};

inline Vector2 operator-( const Vector2 &first, const Vector2 &second ) {
	Vector2 result;
	result.x = first.x - second.x;
	result.y = first.y - second.y;
	return result;
};

inline Vector2 operator*( const Vector2 &first, const float scalar ) {
	Vector2 result;
	result.x = first.x * scalar;
	result.y = first.y * scalar;
	return result;
};

inline Vector2 operator/( const Vector2 &first, const Vector2 &second ) {
	Vector2 result;
	result.x = first.x / second.x;
	result.y = first.y / second.y;
	return result;
};

inline Vector2 operator/( const Vector2 &first, const float scalar ) {
	Vector2 result;
	result.x = first.x / scalar;
	result.y = first.y / scalar;
	return result;
};

inline Vector2 operator*( const Vector2 &first, const Vector2 &second ) {
	Vector2 result;
	result.x = first.x * second.x;
	result.y = first.y * second.y;
	return result;
};

inline float dot(const Vector2 &first, const Vector2 &second) {
	float dp = first.x*second.x + first.y*second.y;
	return dp;
};

inline float RadBetween(const Vector2 &first, const Vector2 &second) {
// dotprod : ABcostheta = a.x*b.x + a.y*b.y
	float dp = first.x*second.x + first.y*second.y;
	dp = dp / (first.length() * second.length());
	return (float) acos(dp);
}

// 3-float vector
class Vector3 {
public:
	Vector3() {};
	Vector3(float _x, float _y, float _z) { x = _x; y = _y; z = _z;};
	float x,y,z;
	inline float length() { return (float) sqrt((x*x)+(y*y)+(z*z)); };
	inline float length2() { return ((x*x)+(y*y)+(z*z)); };			// length squared
	inline void normalize() { float size = length(); x /= size; y /= size; z /= size; };
	Vector3 operator+( Vector3 &second );
	Vector3 operator-( Vector3 &second );
	Vector3 operator*( float scalar );
	inline void operator+=( Vector3 &first) {
		x += first.x;
		y += first.y;
		z += first.z;
	};
	inline void operator-=( Vector3 &first) {
		x -= first.x;
		y -= first.y;
		z -= first.z;
	};
	inline void operator/=( float divisor ) {
		x /= divisor;
		y /= divisor;
		z /= divisor;
	};
	inline void operator*=( Vector3 &second ) {
		x *= second.x;
		y *= second.y;
		z *= second.z;
	};
	inline void operator*=( float scalar ) {
		x *= scalar;
		y *= scalar;
		z *= scalar;
	};
	inline float dot(Vector3 &other) {
		return other.x*x + other.y*y + other.z*z;
	};
};

inline Vector3 Vector3::operator+( Vector3 &other ) {
	Vector3 result;
	result.x = x + other.x;
	result.y = y + other.y;
	result.z = z + other.z;
	return result;
};

inline Vector3 Vector3::operator-( Vector3 &other ) {
	Vector3 result;
	result.x = x - other.x;
	result.y = y - other.y;
	result.z = z - other.z;
	return result;
};

inline Vector3 Vector3::operator*( float scalar ) {
	Vector3 result;
	result.x = x * scalar;
	result.y = y * scalar;
	result.z = z * scalar;
	return result;
};

// integer 2d vector
struct Coord {
	Coord() {};
	Coord(int a,int b) { x = a; y = b; };
//	Coord(CoordF &a) { x = (int) a.x; y = (int) a.y; };
	friend Coord operator*( Coord &first, Coord &second );
	friend Coord operator+( Coord &first, Coord &second );
	friend Coord operator-( Coord &first, Coord &second );
	inline void operator+=( Coord &first) {
		x += first.x;
		y += first.y;
	};
	inline void operator-=( Coord &first) {
		x -= first.x;
		y -= first.y;
	};
	inline void operator/=( int divisor ) {
		x /= divisor;
		y /= divisor;
	};
	inline void operator*=( int scalar ) {
		x *= scalar;
		y *= scalar;
	};
	inline bool operator==( Coord &other) {
		return !operator!=(other);
	};
	inline bool operator!=( Coord &other) {
		if (other.x == x && other.y == y)
			return false;
		return true;
	};
	int x;
	int y;
};

inline Coord operator*( Coord &first, Coord &second ) {
	Coord result;
	result.x = first.x * second.x;
	result.y = first.y * second.y;
	return result;
};

inline Coord operator+( Coord &first, Coord &second ) {
	Coord result;
	result.x = first.x + second.x;
	result.y = first.y + second.y;
	return result;
};

inline Coord operator-( Coord &first, Coord &second ) {
	Coord result;
	result.x = first.x - second.x;
	result.y = first.y - second.y;
	return result;
};

typedef Coord VectorI;
typedef VectorI Coord;
typedef Vector2 CoordF;
typedef Vector2 Vector;

struct Rect {
	Rect() {};
	Rect(Coord p1, Coord p2);
	Rect(int _x, int _y, int _w, int _h) { x = _x; y = _y; width = _w; height = _h; };
	Rect( Rect &other) { x = other.x; y = other.y; width = other.width; height = other.height; };
	Rect &operator=( const Rect &other ) { x = other.x; y = other.y; width = other.width; height = other.height; return *this;};

	int x, y;
	int width, height;
};

struct Rect4 {
	int x0, y0;
	int x1, y1;
};

struct RectF {
	float x, y;
	float width, height;
};

struct Rect4F {
	Rect4F() { };
	Rect4F(float _x0, float _y0, float _x1, float _y1) { x0 = _x0; y0 = _y0; x1 = _x1; y1 = _y1; };
	float x0, y0;
	float x1, y1;

	Rect toRect() { Rect r(int(x0), int(y0), int(x1-x0), int(y1-y0)); return r; };

	void offset(CoordF &offset) { x0 += offset.x; x1 += offset.x; y0 += offset.y; y1 += offset.y; };

	float width() { return x1 - x0; };
	float height() { return y1 - y0; };

	Vector2 size() { return Vector2(width(), height()); };
	Vector2 center() { return Vector2(0.5f*(x0+x1), 0.5f*(y0+y1)); };

	void Validate(); // make sure x0, y0 are <= x1,y1
};

class Polygon2 : public Geometry
{
public:
	Polygon2() { };
	Polygon2(Vector2 *ptlist, int ptsize);

	void addPoint(Vector2 _pt) { pts.push_back(_pt); };
	Rect4F bounds();

	bool IntersectPoint(const Vector2 &pt);	// True if point is inside Polygon
	bool IntersectRect(const Rect4F &rect);	// True if polygon and rectangle overlap

	bool isContainedBy(const Rect4F &container);// True if the polygon is completely within the bounds of the container

	void Rotate(const float angle);				// Rotates around origin
	void Translate(const Vector2 &offset);

	// Returns false if given location is out of bounds
	bool spanX(const float xval, CoordF &result);	// Retrieve polygon min/max span at given X value.
	bool spanY(const float yval, CoordF &result);	// Retrieve polygon min/max span at given Y value. 

	std::vector<Vector2> pts;
};



Rect4F RectF_to_Rect4F(RectF &r);
RectF Rect4F_to_RectF(Rect4F &r);

inline Coord f2i(CoordF &f) { return Coord(int(f.x), int(f.y)); };
inline CoordF i2f(Coord &i) { return CoordF(float(i.x), float(i.y)); };


bool Contains_Rect4F(const Rect4F &outer, const Rect4F &inner);
bool Intersect_Rect(const Rect &a, const Rect &b);
bool Intersect_RectF(const RectF &a, const RectF &b);
bool Intersect_Rect4F(const Rect4F &a, const Rect4F &b);
bool Intersect_Point_Rect(const Coord &a, const Rect &b);
bool Intersect_Point_Rect4(const Coord &a, const Rect4 &b);
bool Intersect_Point_RectF(const CoordF &a,const RectF &b);
bool Intersect_Point_Rect4F(const CoordF &a, const Rect4F &b);
bool Intersect_Point_Poly(const CoordF &a, CoordF *bound, int numbound, Vector offset = Vector(0,0));

bool Intersect_LinePoint(const Coord &a, const Coord &b, const Coord point, const float tolerance = 1.0f);

bool Intersect_LineRect4F(const CoordF &a, const CoordF &b, const Rect4F &rect);
CoordF Find_Intersect_Line_Line(const CoordF &a1,const CoordF &a2,const CoordF &b1,const CoordF &b2, float *u, float *v);

float distance(CoordF &a, CoordF &b);
float distance2(CoordF &a, CoordF &b);

Vector2 Rotate2(Vector2 pt, float angle);							// 2D rotation of a point in plane
Vector3 Rotate3(Vector3 pt, float xang, float yang, float zang);	// 3D rotation of a point in space

#endif