// All global data and functions that are accessible across all plugins/cores are here.
// Each plugin gets a pointer to these from the core when they register. The core itself
// has the actual data/implementations. This allows for better memory heap management across
// DLL boundaries, as well as eliminating the need for linking to an external core.dll.

// And yes, this is very C-style using function pointers instead of deriving from an abstract interface. The idea is the same, but
// some of this is very very old code that hasn't been updated in years.


#pragma once
#ifndef _GLOBALS_H
#define _GLOBALS_H

#include "..\core\common.h"
#include <vector>

class WMThreadPool;
class OutputStore;
class Device;
class WMPacket;

typedef Device* (DMakePtr)();
typedef void (DKillPtr)(Device* ptr);
typedef WMPacket* (PackMakePtr)();
typedef void (PackKillPtr)(WMPacket* ptr);
typedef int  (UIPropDlgType)(Device *ptr);
typedef bool (UINameDlgType)(Device *ptr);
typedef bool (UIAboutDlgType)(Device *ptr);
typedef void (UIShowDeviceEdit)(Device *ptr);

// Each device contains a DEV_DATA structure. The WM Core uses this structure to manage
// the device (load, create, and destroy instances of the device class)
struct DEV_DATA {		// Entry in device tag-library (1 entry per device derivative)
	DEV_DATA() { nametag[0] = NULL; maker = NULL; killer = NULL; nullterm = 0;};
	char nametag[4];
	char nullterm;
	DMakePtr *maker;	// fpointer to create a device instance (in device's heap)
	DKillPtr *killer;	// fpointer to destroy a device instance (in device's heap)
};

class HField;
class WMPacket;

const int PROGRESS_SCALE = 100;

typedef Device* (*FrozenPreviewPtr)(Device *ptr, void *doc);
void SetFrozenPreviewCallback(FrozenPreviewPtr ptr, void *data);

// Global (shared) callbacks and application information accessible by device plugins
// if desired.

struct WMSharedDeviceData {
	UIPropDlgType *ui_prop_callback;
	UINameDlgType *ui_name_callback;
	UIAboutDlgType *ui_about_callback;
	void *reportdata;
	bool showParams;
};

struct WMSharedWorldData {
	FrozenPreviewPtr	devpreview;	// callback for app to modify device to be previewed...
	void *frozen_data;				// data ptr stored by application
	int previewsize;
};

struct WMSharedUIData {
	UIShowDeviceEdit *ShowInOverview;	// Call to display the passed device in Layout:Overview:Device edit
};

////////////////////////////////////////////////////////
// World Machine Global (Shared) Access Functions
////////////////////////////////////////////////////////
// These are supplied by whatever WM program core is running.

// these macros work if the global pointer is called WMGlobal
#define WMAlloc(x) WMGlobal->Alloc(x)
#define WMFree(x) WMGlobal->Free(x)
#define WMGetPacket(x) WMGlobal->GetPacket(x)

class DeviceWorld;

const int GLOBAL_PATH_LENGTH = 2048;

enum READUNIT_TYPE { READUNIT_AUTO = -1, READUNIT_WM, READUNIT_KM };
enum WRITEUNIT_TYPE { WRITEUNIT_AUTO, WRITEUNIT_FORCE_M, WRITEUNIT_FORCE_KM, WRITEUNIT_FORCE_WM };

struct WMGlobalStruc {
// Memory allocation -- MUST use these functions if the core will delete the data.
// The WM Core will throw a WMException-class exception if the allocation fails.
	void *(*Alloc)(int size);		// Alloc from the core's heap
	int (*Free)(void *pointer);		// Free to the core's heap

//	void (*EvictPackets)(unsigned int memreq_size);
	void (*ManagePacketAllocation);
	void (*CacheEvictOutputs)();		// Force active world to drive to cache all currently cacheable device build-final packets
	void (*CacheDeviceOutput)(Device *dev, int outport, WMPacket *map);

// Request a new packet of a specific type
	WMPacket *(*GetPacket)(const int id);
// Free the packet back to parent app's pool
	void (*FreePacket)(WMPacket * wmp);

// Request a new device of the specified type
	Device *(*GetDevice)(const char* id);
// Get a worldfile-unique identifier for a device
	int (*GetDeviceUniqueIdentifier)(void);

// Device Registry
	bool	 (*RegisterDeviceType)(DEV_DATA entry);	// Register a Device
	DEV_DATA (*GetDeviceRegistry)(const char* tag);	// Retrieve entry for a given Device tag, if avail
    void (*ClearTags)();							// Clear all registered sprocket types

// Device callback functionality. Allows devices to request the owner program perform some operation
	bool (*PreviewWorld)(DeviceWorld *world);
	bool (*BuildWorld)(DeviceWorld *world);
	bool (*PreviewDevice)(Device *dev);

// Retrieve the current colortable for HF display
	void (*RetrieveColorTable)(dword **colors, int *numcolors);

// UI Callbacks, for converting to/from WM or KM units
	float (*ReadUnit)(const char *str, READUNIT_TYPE default_dimension);
	float (*ReadAltitudeUnit)(const char *str);
	std::string (*WriteUnit)(float wm_unit, WRITEUNIT_TYPE auto_metric_switching);
	std::string (*WriteAltitudeUnit)(float wm_unit, WRITEUNIT_TYPE auto_metric_switching);

// Debugging (logging) functions
	void (*Log)(int channel, const char *message);

// Specific shared areas
	WMThreadPool	*threadpool;	// Thread pool created by host application
	WMSharedDeviceData	gdevice;
	WMSharedWorldData	gworld;
	WMSharedUIData		gui;

	char	home_path[GLOBAL_PATH_LENGTH];	// path to executable, DLLs, help
	char	data_path[GLOBAL_PATH_LENGTH];	// path to INI, resource folder
	char	doc_path[GLOBAL_PATH_LENGTH];	// path to default document folder, examples, etc
	char	macro_path[GLOBAL_PATH_LENGTH];	// path to macro library
	char	preset_path[GLOBAL_PATH_LENGTH];// path to preset library
	char	temp_path[GLOBAL_PATH_LENGTH];	// path to temp folder
	bool	core_registered;	// are we running under a registered WM core
// ** DATA **
	float	km_wm_relation;		// # of km in 1 WM unit
	int		packet_count;		// Count of total # of current packets that have been allocated

// Experimental Help for plugin user interface code that needs to pass messages to the main WM window
//	HWND	home_window;
};

// SizeData structure is used by heightfields and devices to specify the size of heightfields
// in both actual physical data size, and its world extents.
struct SizeData {
	SizeData() { };
	SizeData(Coord &_size, CoordF &_scalesize) { size = _size; scalesize = _scalesize; CalcScale(); };

	void CalcScale() {
		detail_scale = float(size.x) / scalesize.x; // X and Y are guarenteed to follow the same scaling
	};
	Coord size;			// Total size of HF in elements
	CoordF scalesize;	// "Scale" size of HF: Size it takes up in worldspace units
	float detail_scale;	// The linear transform from scalesize to size (ie units are pixels/WMu)
	float vert_scale;	// The vertical scale of the HF
};

extern WMGlobalStruc *WMGlobal;

#endif