#pragma once

class HField;
class WMPacket;
class BMPacket;

#include <string>
#include "common.h"

enum FI_FORMAT { FI_NONE, FI_TERRAGEN, FI_RAW, FI_RAW16, FI_RAW32, FI_BT, FI_BMP, FI_TGA, FI_TIFF, FI_PNG, FI_HFZ};

class HFieldExporter
{
public:
	HFieldExporter();
	HFieldExporter(HField *hf);
	~HFieldExporter(void);

	void setMetadata(const char *txt) { metadata = txt; };
	void setRGB(bool _rgb) { rgb = _rgb; };

// RGB or HF Exporters
	bool SavePNG(const char *fname);

//// HF only exporters
// 8bit
	bool SaveTGA(const char *fname, bool colorize = false);	// Save as a 24bit TGA file
	bool SaveRAW(const char *fname);	// Save as a raw, 8bit int dump
	bool SaveBMP(const char *fname, bool colorize = false);
// 16bit+
	bool SaveRAW16(const char *fname);	// Save as a raw 16bit int dump
	bool SaveRAW32(const char *fname);	// Save as a raw, 32bit FP dump
	bool SaveTGA16(const char *fname);
	bool SaveEXR(const char *fname);
	bool SaveTIFF16(const char *fname);
	bool SavePGM(const char *fname);
// scale-carrying
	bool SaveBT(const char *fname);
	bool SaveTer(const char *fname, bool addpadding = true);	// Save in Terragen format
	bool SaveLev(const char *fname);
	bool SaveHFZ(const char *fname);	// HF2/HFZ format

private:
	// export data
	bool hasMetadata() {return !metadata.empty(); };
	bool rgb;
	std::string metadata;

	// export data
	HField *hf;
};

class HFieldImporter
{
public:
	HFieldImporter() { has_scale = false; };
	~HFieldImporter() { };

// Import Functionality
	void setRGB(bool _rgb) { rgb = _rgb; };

	bool hasMetadata() { return !metadata.empty(); };
	std::string &getMetadata() { return metadata; };

	bool hasScaleData() { return has_scale; };
	Vector2 getScaleData() { return scalerange; };

// Master load function, will determine filetype from extension.
// can use this in place of the below if you want
	WMPacket *LoadFile(const char *fname);

// Bitmap to HF Converter (luminance convert)
	HField		*ConvertToHF(BMPacket *src);
// HF to Bitmap Converter (greyscale)
	BMPacket	*ConvertToBMP(HField *src);

// RGB or HField Importers
	WMPacket	*LoadBMP(const char *fname);
	WMPacket	*LoadTGA(const char *fname);
//	WMPacket	*LoadTIF(const char *fname);
//	WMPacket	*LoadJPG(const char *fname);
	WMPacket	*LoadPNG(const char *fname);
	WMPacket	*LoadTIFF(const char *fname);

// HF only importers
	HField		*LoadTer(const char *fname);
	HField		*LoadBT(const char *fname);
	HField		*LoadHFZ(const char *fname);

	HField		*LoadRAW(const char *fname);
	HField		*LoadR16(const char *fname);
	HField		*LoadR32(const char *fname);

private:
	bool AddScaleMetadata();
	// import data
	bool rgb;
	std::string metadata;

	bool has_scale;
	CoordF scalerange;
};


FI_FORMAT GetFileType(const char *name);
void FlipPacket(WMPacket *wmp);