#pragma once
#include "device.h"
#include "Plugin_Header.h"

// Derive from these classes to specify a Generator or Output device

class Generator : public Device  
{
public:
	Generator();
	virtual ~Generator();

	virtual int GetDeviceFamily() { return TYPE_GENERATOR; };
	virtual dword Color() { return (0x068b12c); }; 

	virtual bool isMaskable() { return true; };
	virtual bool hasOrigin() { return true; };
	virtual bool isExplorable() { return getOriginType() != 1; };	// is now Explorer-stable in Pro mode
	virtual bool hasLocalOverride() { return true; };
protected:
	virtual void MaskOutput(WMPacket *map, int slotnum, PSTATE preview);

};

// OutputSet: the inputs to the output device necessary for it to function
struct OutputSet {
public:
	OutputSet() { };
	OutputSet(WMPacket *first) { set(0, first); };
	OutputSet(OutputSet const& other);
	OutputSet &operator= (const OutputSet & other);

	int num() { return inputs.size(); };
	void set(int i, WMPacket *pack) { if (!pack) return; if (i >= inputs.size()) { inputs.resize(i+1, NULL); } inputs[i]->Release(); inputs[i] = pack; }; //pack->AddRef(); };
	WMPacket *get(int i) { if (i < inputs.size()) { return inputs[i]; } return NULL; };

	// Remove contents WITHOUT deallocation!!!
	void clear() { inputs.clear(); };
	// Remove and deallocate contents
	void cleanup() { for (int i = 0; i < inputs.size(); i++) { inputs[i]->Release(); } inputs.clear(); };
private:
	std::vector<WMPacket *> inputs;
}; 

class  Output : public Device  
{
public:
	Output();
	virtual ~Output();

	virtual void doClone(Device *other);			// Copy data from other to this
	virtual bool LoadSession(std::istream &in);
	virtual bool SaveSession(std::ostream &out);

	enum OUTPUT_MODE { OUTPUT_NORMAL, OUTPUT_TILED};

	virtual int GetDeviceFamily() { return TYPE_OUTPUT; };
	virtual dword Color() { return (0x03030cc); };

	virtual bool doOutput(OutputSet &outset, OUTPUT_MODE output_mode) = 0;

	virtual bool RunUI(); // Output RunUI always returns false; its impossible to invalidate since we dont operate on it

	virtual ACTIVATE_RESULT ActivateOnNeed(BuildContext &bi);
	
	virtual bool allowTileBlend() { return true; };
	virtual void Reset(PSTATE which_build = P_ALL);// Reset part to unprocessed state, destroying any packets residing inside
	virtual WMPacket *getDefaultOutput(PSTATE preview);

	virtual bool isExporting(OUTPUT_MODE mode) = 0;	// Return false if output is view-only or is otherwise not currently exporting (this helps tiled build and others save time)
	virtual bool isMaskable() { return false; };

	OutputSet &RetrieveOutputContents();			// OutputSet contents are always final build

protected:
// OutputBuffer and OutputContents store the result of working on 
	// Store contents of outputset and optional preview packet. If any are not stored they are managed (deleted) appropriately
	void	   StoreOutputContents(OutputSet &final_oset, WMPacket *preview_packet, PSTATE preview);  // preview_packet can be NULL if no special preview is created
private:
	WMPacket * RetrieveOutputBufferConst(PSTATE pstate);
	WMPacket   *output_buffer[3];	// Final output result packet -- for viewing
	OutputSet	final_outputset;	// The set of P_FINAL inputs -- for exporting (need all input not just output preview)
};