#pragma once

#include "HField.h"

// Encapsulates an optional mask input. If a heightfield is specified, it returns the value from
// the heightfield.
// otherwise it returns a constant value.
class InputWrap
{
public:
	InputWrap(HField *hf, const float default_val = 0.0f);

	void SetDefault(float _val) { val = _val; };
	inline float value(const Coord &loc) const;
	inline float operator[](const Coord &loc) const;
	inline float operator[](const int index) const;
protected:
	HField *map;
	float val;
};

inline InputWrap::InputWrap(HField *hf, const float default_val) {
	map = hf;
	val = default_val;
};

float InputWrap::value(const Coord &loc) const {
	if (map)
		return map->value(loc);
	else
		return val;
};

float InputWrap::operator[](const Coord &loc) const {
	if (map)
		return map->value(loc);
	else
		return val;
};

float InputWrap::operator[](const int index) const {
	if (map)
		return (*map)[index];
	else
		return val;
};

class InputWrapMult : public InputWrap {
public:
	InputWrapMult(HField *hf, const float mult_val) : InputWrap(hf, mult_val) { };
	inline float value(const Coord &loc) const;
	inline float operator[](const Coord &loc) const;
	inline float operator[](const int index) const;
};

float InputWrapMult::value(const Coord &loc) const {
	if (map)
		return map->value(loc) * val;
	else
		return val;
};

float InputWrapMult::operator[](const Coord &loc) const {
	if (map)
		return map->value(loc) * val;
	else
		return val;
};

float InputWrapMult::operator[](const int index) const {
	if (map)
		return (*map)[index] * val;
	else
		return val;
};

