#pragma once
#include "WMPacket.h"

// Parameter Packet.. transfers a single parameter from one thing to another
// Scalar Packet transfers a value
#include "Parameter.h"

class ParamPacket : public WMPacket
{
public:
	ParamPacket(void);
	~ParamPacket(void);
	ParamPacket(const ParamPacket &other);		// copy construction
	ParamPacket &operator=(const ParamPacket &other);

	virtual int getHeapUsage() { return 0;	};

	virtual int getType() { return WMP_parameter; };
	virtual char *getTypeName() { return "Parameter"; };
	virtual WMPacket *Clone();
	virtual bool Load(std::istream &input) { assert(false); return false; };	
	virtual bool Save(std::ostream &output) { assert(false); return false; };	

	virtual dword Color() { return 0x0ff0000; };

	int paramType() { return param.type; };
	VAL &paramVal() { return param.value; };	// reference to parameter value

	Parameter &getParam();
	void setParam(const Parameter &p);

protected:
	virtual void Kill();		// Call to delete its own type
	virtual void clearContents() { };

	Parameter param;			// the included parameter
};

class ScalarPacket : public WMPacket 
{
public:
	ScalarPacket(void);
	~ScalarPacket(void);
	ScalarPacket(const ScalarPacket &other);		// copy construction
	ScalarPacket &operator=(const ScalarPacket &other);

	virtual int getHeapUsage() { return 0;	};

	virtual int getType() { return WMP_scalar; };
	virtual char *getTypeName() { return "Scalar"; };
	virtual WMPacket *Clone();
	virtual bool Load(std::istream &input) { assert(false); return false; };	
	virtual bool Save(std::ostream &output) { assert(false); return false; };	

	virtual dword Color() { return 0x0770000; };

	float val() { return scalar; };
	void setval(float _newval) { scalar = _newval; };

protected:
	virtual void Kill();		// Call to delete its own type
	virtual void clearContents() { };

	float scalar;
};

// Accessor inline function to provide a safer way to cast WMPacket to ParamPacket
// in debug mode it will assert if the packet is not a ParamPacket; in release you pay nothing
// framework should never allow this anyways, but it never hurts to be careful!
inline ParamPacket *PARAM(WMPacket *pack) { 
	if (pack) {
#ifdef _DEBUG
		bool ok = pack->getType() == WMP_parameter;
		assert(ok);
		if (!ok)
			return NULL;
#endif
	}
	return (ParamPacket*) pack; 
};

inline ScalarPacket *SCALAR(WMPacket *pack) { 
	if (pack) {
#ifdef _DEBUG
		bool ok = pack->getType() == WMP_scalar;
		assert(ok);
		if (!ok)
			return NULL;
#endif
	}
	return (ScalarPacket*) pack; 
};
