#ifndef _PLUGIN_H
#define _PLUGIN_H
#include "Globals.h"

// Your DLL will export this value. It allows the WM framework to only load plugins
// that were built against the current headers
const int WM_PLUGIN_HEADER_VERSION = 4;			// 1: Initial Public v1.0 Release
												// 2: WM 1.1 / WM1.25 Release
												// 3: WM Pro / WM2.0 Release
												// 4: WM 2.1 Release

// Each device in a plug has an entry. See the example files for details of usage.
struct DevPluginStruc {
	DevPluginStruc() { deprecated = false; type = 0; name[0] = 0; };
	DEV_DATA lifedata;			// Data & Ptrs to life management functions
	char	name[64];			// Name of this plugin
	int		type;				// What category does this device fit into? (See Dev Type Constants Below)
	int		datatype;			// What type of data does this work on (See WMPacket constants). Can be multiple | together
	bool	deprecated;			// Set to true if this device shouldnt show up in the listing anymore
	char	reserved[64];		// room for future expansion...
};


// ******* The following functions are REQUIRED for both C and C++ interfaces! *******
// (Required) All plugins need to implement this function; WM will not allow load without it
typedef void (*SetupGlobalAccessPtr)(WMGlobalStruc *global);
// (Required) DLL reports version of WM core that the plugin is built against.
//            Required for WM to do version-error checking of plugins. This function should simply return WM_PLUGIN_HEADER_VERSION
typedef	int  (*GetHeaderVersionPtr)();
// (Optional) Plugins should implement this if one or more of the devices are registered-only
typedef int (*RegOnlyPtr)();

// *********************************************************************************
// (REQUIRED) IMPLEMENT ONE OF THE BELOW IN YOUR DLL.
// ******** C++ DEVICE ITERATION NTERFACE 
// C++ iteration function to implement for Device Plugins
typedef	bool (*GetDevPluginPtr)(const int index, DevPluginStruc *result);
// ******** C DEVICE ITERATION NTERFACE 
//  A C++ "holster" module has been created to allow you to create your DLLS against a C-style interface, allowing for
// greater compiler and language support.
//typedef	bool (*GetDevPluginPtr)(const int index, DevPluginCStyleStruc *result);
// *********************************************************************************


// Device Type Constants
// Heightfield devices
const int TYPE_GENERATOR = 1;
const int TYPE_OUTPUT = 2;
const int TYPE_COMBINER = 3;
const int TYPE_FILTER = 4;
const int TYPE_NATURAL = 5;
const int TYPE_SELECTOR = 6;
const int TYPE_UTILITY = 7;
const int TYPE_MISC = 8;
const int TYPE_CONVERTER = 9;
// Param/Scalar devices
const int TYPE_SCALAR_OPERATOR = 16;
const int TYPE_PARAM		   = 17;

// Shape plugins
struct ShapePluginStruc {
	ShapePluginStruc() { deprecated = false; type = 0; name[0] = 0; };
	DEV_DATA lifedata;			// Data & Ptrs to life management functions
	char	name[64];			// Name of this plugin
	int		type;				// What category does this shape fit into? currently all 0
	bool	deprecated;			// Set to true if this device shouldnt show up in the listing anymore
	char	reserved[64];		// room for future expansion...
};

// (Required) Function to implement for Shape Plugins
typedef	bool (*GetShapePluginPtr)(const int index, ShapePluginStruc *result);


#endif