#pragma once

// The port class acts as the mechanism through which data moves into and out of the device. Ports, not devices, are wired together.
// Each port can also store exactly one data packet.

#include "common.h"
#include <vector>

class Device;
class WMPacket;
enum PSTATE;

const int PORT_NAME_LEN = 24;
// Base class Port
class Port {
public:
	Port();
	virtual ~Port(void);

	void SetOwner(Device* _o) { owner = _o; };
	Device *Owner() const { return owner; };
	virtual Port *Target() const { return target; };	// The other port
// Return the other device linked through this port
	virtual Device *Other() const { if (target) { return target->Owner(); }; return NULL; };

// Outside code can call these to connect or disconnect ports
	virtual bool MakeLink(Port *other);
	virtual bool BreakLink(Port *other = NULL);

// the type of packets that the port accepts (see WMPacket.h for enumations)
	bool AcceptsType(int type);				// test to see if this port will accept this type
	int GetType() { return data_type; };
	void SetType(int _new) { data_type = _new; };

// Data transfer functions
	void ClearData(PSTATE pstate);
	void StoreData(PSTATE pstate, WMPacket *wmp);
	WMPacket *RetrieveData(PSTATE pstate, bool transfer);

// Port world viewport functions
	void SetName(const char *_name);
	char *GetName() { return name; };

	Coord GetLoc(); 

	// flag port as either normal (=0) or special (=1)
	void SetFlag(int state) { flag = state; };
	int GetFlag() const { return flag; };

protected:
	friend class OutputPort;
	virtual void SetTarget(Port *newlink, Port *oldlink = NULL);
	char name[PORT_NAME_LEN];
	int data_type;			// what sorts of data can be taken at this port
	Port *target;
	Device *owner;
	int flag;				// is this port normal or special?

	WMPacket *data[3];	
};

// 11/01/05 : Considering extending to OutputPort class for splitter-less networks
// Output port is the same as a port except that it has multiple output targets
class OutputPort : public Port {
public:
	OutputPort();
	virtual ~OutputPort(void);
//	OutputPort(const OutputPort &other);			// copy construction
//	OutputPort &operator=(const OutputPort &other);

// OutputPort Specific Functions
	void StoreAndDistributeData(PSTATE pstate, WMPacket *wmp);
	void DistributeData(PSTATE pstate);

// The (first) port we're connected to is default..
	virtual Port *Target() const;
	virtual Device *Other() const;

// Return a specific output target
	Port *Target(int index) const;
	Device *Other(int index) const;

	int numLinks() { return int(outlist.size()); };
	virtual bool MakeLink(Port *other);
	virtual bool BreakLink(Port *other = NULL);
protected:
	virtual void SetTarget(Port *newlink, Port *oldlink = NULL);
private:
	std::vector<Port*> outlist;
	int out_ready;
};