#pragma once

#include <vector>
#include "parameter.h"

struct PresetName {
	char tag[4];				// tag of device owner
	std::string presetname;		// filename of preset
};

struct PresetData {
	char tag[4];				// tag of device owner
	ParameterSet params;
};

// Encapsulates the functionality of presets for devices.
// It does NOT store the actual data of the preset.. however it manages scan/load/store of them
// Used by GenericDlg, and YOUR dialog that wants to use device presets
class PresetManager
{
public:
	PresetManager(void);
	~PresetManager(void);

	void SetPresetBase(Device *_owner, const char *tag, const char *WMdir);

	bool ScanPresets();		// create list of presets for a given tag
	bool LoadPreset(const PresetName &name, PresetData &preset);		// load a preset from disk
	bool SavePreset(const PresetName &name, PresetData &preset);	// save a preset to disk
	bool DelPreset(const PresetName &name);		// remove a preset from disk

	int			numPresets()				{ return presetlist.size(); };
	PresetName &getPreset(int index)		{ assert(index >= 0); assert(index < presetlist.size()); return presetlist[index]; };
	void		addPreset(const char *str);
protected:
	bool ready;							// has basedir been set?
	Device *owner;
	char tag_id[4];						// Tag ID for the current list of presets 
	std::string basedir;
	std::vector<PresetName> presetlist;
};
