#pragma once

#include "common.h"
#include "HField.h"
class HField;
class BMPacket;

/*
A generic class for resampling a bitmap or heightfield at a new dimension, using box scaling
Instead of deriving from it, all the basic ones I need are included in the base class for convenience.
An extended version can be derived to add new rescaling if you need it.
*/
class Rescaler
{
public:
	Rescaler(void);
	~Rescaler(void);

	static Coord LimitSize(const Coord &size, int limit);
	static Coord LimitSize(const Coord &size, const Coord &limit); // expand until we maximally fill limit

	static HField *RescaleHF(HField *input, Coord  dims);		// Rescale input heightfield to the given dimensions and return new one
	static HField *RescaleHF(HField *input, SizeData &dims);	// Rescale input heightfield to the given dimensions and return new one

	static BMPacket *RescaleBMP(BMPacket *input, Coord  dims);		// Rescale input bitmap to the given dimensions and return new one
	static BMPacket *RescaleBMP(BMPacket *input, SizeData &dims);	// Rescale input bitmap to the given dimensions and return new one

	static void Nearest2(HField &src, HField &dest);
	static void Bilinear2(HField &src, HField &dest);
	static void Bicubic2(HField &src, HField &dest);
	static void BilinearBMP(BMPacket &src, BMPacket &dest);

	static void FractalInterp(HField &src, HField &dest, const float smoothness = 0.5f);
protected:
    
};