#pragma once
#include "wmpacket.h"

#include "hfield.h"
#include "bmpacket.h"

class HField;
class BMPacket;

/*

TerrainCombo contains both a Heightfield and a RGB image. V1.0 requires they be the same
resolution, although there is no reason they couldn't differ.

This is an internal datatype, as there really is no reason to pass them around combined like
this, at least for now.

*/
class TerrainComboPacket :
	public WMPacket
{
public:
	TerrainComboPacket(void);
	~TerrainComboPacket(void);

	virtual int getType() { return WMP_combo; };
	virtual char *getTypeName() { return "Terrain Combo"; };
	virtual dword Color() { return 0x00ff0000; };

	virtual int getHeapUsage() { if (!hf || !bmp) return 0; return hf->getHeapUsage() + bmp->getHeapUsage(); };

	virtual WMPacket *Clone();			// Return a copy of this WMPacket-derived packet
	virtual bool Load(std::istream &input);	
	virtual bool Save(std::ostream &output);	

	bool AssignHF(HField *_hf);
	bool AssignBMP(BMPacket *_bmp);

	HField *getHF() { return hf; };
	BMPacket *getBMP() { return bmp; };
private:
	virtual void Kill();		// Call to delete its own type
	virtual void clearContents();

	HField *hf;
	BMPacket *bmp;
};

// Safe Accessor inline function to provide a safe way to cast WMPacket to HField
// in debug mode it will assert if the packet is not a HF
inline TerrainComboPacket *TC(WMPacket *pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_combo;
		if (!ok)
			return NULL;
	}
	return (TerrainComboPacket*) pack; 
};


inline TerrainComboPacket *GetNewTerrainCombo() {
	TerrainComboPacket *tc = TC(WMGlobal->GetPacket(WMP_combo));
	return tc; 
};