#pragma once

/*

WMThreadPool : manage program-level threads, handing out threads to clients as requested.

On Program Start, WM will initialize the threadpool. Clients may run a worker thread by simply requesting one from the pool with runThread()...
if the threadpool is empty, runThread will return false.

Internally, each thread runs its own procedure, idling until handed a client function. When that client function returns, it goes back to an idle.

*/

typedef void* WMThreadDataP;
typedef UINT (*WMThreadProc)(WMThreadDataP userdata);

class WMThreadPool;

struct WMThreadInternalStruc {
	WMThreadInternalStruc() { owner = NULL; alive = false; idle = false; userproc = NULL; userdata = NULL; thread_event = NULL; };
	WMThreadPool *owner;	// Pointer to the thread pool

	volatile bool alive;	// is there a thread associated with this data?
	volatile LONG idle;		// set to true by the thread if its waiting for a new client function
	WMThreadProc	 userproc;	
	WMThreadDataP	 userdata;
	HANDLE	thread_event;	// signal'd by the thread pool when a new task is recieved.
};

const int WM_THREADMAX = 16;

class WMThreadPool
{
public:
	bool startThreadPool(int num_threads, int thread_priority);
	bool endThreadPool(bool return_immediately = false);

	int maxThreads() { return max_threads; };
	int activeThreads();
	int availableThreads() { return maxThreads() - activeThreads(); };

	virtual bool beginLock() = 0; // if you have several things to do (like start multiple threads)
	virtual bool endLock() = 0;	  // ...

	// Request a thread from the thread pool to run with the given procedure and data.
    virtual bool runThread(WMThreadProc userproc, WMThreadDataP userdata) = 0;

protected:
	virtual void startThread(WMThreadInternalStruc *thread_control, int thread_priority) = 0;

	int max_threads;
	int thread_priority;
	WMThreadInternalStruc pool[WM_THREADMAX];

	HANDLE	eventQuit;
	HANDLE	eventDispatch;

	WMThreadPool(void);
	~WMThreadPool(void);
};
