/*

  Bitmap version 2.0 class

  a bitmap is now purely an image data repository.. it now has no knowledge about
  drawing itself

  */

#ifndef _BITMAP_H
#define _BITMAP_H

#include "common.h"
//#include "gfx_def.h"

class istream;
class ostream;

// Change this define whenever disk format changes
#define BITMAP_VERSION		4

enum IMG_DAT_TYPE { RGB16_555, RGB16_565, RGB24, RGBA32, ALPHA8 };

const int FLEN = 64;

inline bool IsRGB(IMG_DAT_TYPE a) { return (a <= RGBA32); };

// Bitmap should have knowledge of rotation and its ability in that regard
class Bitmap {
public:
// Housekeeping
	Bitmap();
	~Bitmap();
	Bitmap( Bitmap &other);
	Bitmap &operator=( const Bitmap &other );

// Native load/save functions, for individual or stream loading
	void CreateBlank(Coord size, IMG_DAT_TYPE format); // Initialize bitmap
	bool Load(const char *fname);	// Load a .gfx file from disk
	bool Save(const char *fname);	// Save this bitmap to a .gfx file
	bool Load(std::istream &in);	// Load a .gfx file from an input stream
	bool Save(std::ostream &out);	// Save this bitmap to an output stream

// Import/Export functions
	bool LoadTGA(const char *fname, IMG_DAT_TYPE convert = RGB16_565);	// load a .TGA from disk
	bool LoadBMP(const char *fname, IMG_DAT_TYPE convert = RGB16_565);	// load and optionally convert a BMP
	bool SaveTGA(const char *fname); // Save the image only to a .tga file

// Manipulation functions for changing the image data contained here
	void ConvertRGB(IMG_DAT_TYPE new_type);	// General purpose intra-RGB format converter
	IMG_DAT_TYPE GetFormat() { return data_type; };
	bool MakeFromBuffer(Coord size, IMG_DAT_TYPE format, void *input); // **NOTE: input buffer turned over to bitmap to manage!

// Public data and access functions
	char *Getname() { return name; };	// Return image name
	byte *Getdata() { return data; };			// Return image data pointer
	byte *GetdataRow(int row) { return data + psize()*gwidth*row; };
	int GetSize() { return size; };
	int width, height;		// Real size of image data per cell
	int gwidth, gheight;	// Total Width and Height of all cells in this bitmap

	dword pixel32(int x, int y);	// Grab 32bit color value at location
	int	psize() { // return pixel size
		switch (data_type) {
		case RGBA32:
			return 4;
		case RGB24:
			return 3;
		case ALPHA8:
			return 1;
		default:
			return 2;
		}
	};

protected:
/////////////////////
	virtual void clear();		// Deletes all data for this object
	virtual void clone(const Bitmap &other); // Makes a deep copy of the other bmp.
	int size;			// Size of data array IN BYTES
	byte *data;			// Pointer to data array
	IMG_DAT_TYPE data_type;	// Type of image data.
	char name[FLEN];	// Name of this bitmap in library
};

#endif