// Common include file
// for global declarations, etc.
#ifndef _COMMONDEF_H
#define _COMMONDEF_H

#pragma warning (disable : 4305)
#define WIN32_LEAN_AND_MEAN

#ifndef _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE
#endif

#define PI 3.141592653589793

#include <assert.h>
#include <math.h>
#include <iostream>

extern std::ofstream logfile;

// These error and log functions need to always be available
void LOG_DEBUG(int channel, const char *text);	// use for debugging purposes. Will direct to the debug console. The older messages are based upon this function:

void  LOG_WRITE(const char *a);
#define LOG_ERROR(a)		{ LOG_WRITE(a); }		
#define LOG_WARNING(a)		{ LOG_WRITE(a); }

#ifndef byte
typedef unsigned char byte;
#endif

#ifndef word
typedef unsigned short word;
#endif

#ifndef dword
typedef unsigned int dword;
#endif

#define USE_x64_COMPATIBLE
#include <intrin.h>

// Neat pow2 functions from flipcode .. wish I'd thoughta them
inline bool IsPow2(int num) {
	return (num && !(num & (num-1)));
}

// Returns next power of two, or num if num == pow2;
inline int NextPow2(int num)
{
    unsigned int RetVal = 1;

    if (IsPow2(num))
		return num;

#ifdef USE_x64_COMPATIBLE
	DWORD val = 0;
	_BitScanReverse(&val, num);
	++val;
	RetVal <<= val;
#else
    __asm    
    {    
        xor ecx, ecx
        bsr ecx, num
        inc ecx
        shl RetVal, cl
    }
#endif
    return (RetVal);
}

// Returns previous power of two, or num if num == pow2;
inline int PrevPow2(int num)
{
    unsigned int RetVal = 1;

#ifdef USE_x64_COMPATIBLE
	DWORD val = 0;
	_BitScanReverse(&val, num);
	RetVal <<= val;
#else
    __asm    
    {    
        xor ecx, ecx
        bsr ecx, num
        shl RetVal, cl
    }
#endif
    return (RetVal);
}

// returns the exponent of the greatest power of two equal-to or less than num
// ie GetPower(2) returns 1, GetPower(16) returns 4.
inline int GetPower(int num) {
	DWORD power = 0;
#ifdef USE_x64_COMPATIBLE
	if (_BitScanReverse(&power, num))
		return power;
	else
		return 0;
#else
	__asm {
        xor ecx, ecx
        bsr ecx, num
		mov power, ecx;
	}
	return power;
#endif
}

union INTORFLOAT {
	int i;
	float f;
};

// neat fast float-to-int function from Game Programming Gems 2
inline int ffi(float val) {
#ifdef USE_x64_COMPATIBLE
	return int(val);
#else
	INTORFLOAT	n, bias;
	n.f = val;
	bias.i = ((23 + 127) << 23) + (1<<22);
	n.f += bias.f;
	n.i -= bias.i;
	return n.i;
#endif
}

inline float degree2rad(float ang) {
	return float(ang * (PI / float(180.0)));
};

inline float rad2degree(float rad) {
	return float(rad * (180.0 / PI));
};

#include "Geometry.h"

inline byte _r(dword c) { dword *cptr = &c; byte *bptr = (byte*) cptr; return bptr[0];}; 
inline byte _g(dword c) { dword *cptr = &c; byte *bptr = (byte*) cptr; return bptr[1];}; 
inline byte _b(dword c) { dword *cptr = &c; byte *bptr = (byte*) cptr; return bptr[2];}; 
inline byte _a(dword c) { dword *cptr = &c; byte *bptr = (byte*) cptr; return bptr[3];};

// Bezier spline interpolation
template <class T> T spline_bezier(T a, T b, T c, T d, float u) {
	float omu = 1.0f - u;
	float omu_square = omu * omu;
	float omu_cubed = omu_square * omu;
	float cubed = u * u * u;
	float squared = u * u;
	return (a * (omu_cubed) +
		    b * (omu_square * 3 * u) +
			c * (omu * 3 * squared) +
			d * cubed);
};

template <class T> T spline_bezier_firstder(T a, T b, T c, T d, float u) {
	float omu = 1.0f - u;
	float omu_square = omu * omu;
	float omu_cubed = omu_square * omu;
	float cubed = u * u * u;
	float squared = u * u;
	return (a * (-3.0f * omu_square) +
		    b * (9.0f * squared - 12.0f*u + 3) +
			c * (-9.0f * squared  + 6.0f*u) +
			d * 3.0f * squared);
};


// Catmul-Rom spline interpolation
template <class T> T spline_catmull(T a, T b, T c, T d, float u) {
	float cubed = u * u * u;
	float squared = u * u;
	return (a * (-0.5f * cubed + squared - 0.5f*u) +
		    b * (1.5f * cubed - 2.5f*squared + 1.0f) +
		    c * (-1.5f * cubed + 2.0f*squared + 0.5f*u) +
		    d * (0.5f*cubed - 0.5f * squared));
};

// B-Spline spline interpolation
template <class T> T spline_bspline(T a, T b, T c, T d, float u) {
	const float onesixth = 1.0f / 6.0f;
	const float onethird = 1.0f / 3.0f;

	float omu = 1.0f - u;
	float omu_cube = omu * omu *omu;
	float cubed = u * u * u;
	float squared = u * u;
	return (a * (onesixth * omu_cube) +
		    b * (0.5f * cubed - squared + 2.0f*onethird) +
		    c * (-0.5f * cubed + 0.5f*squared + 0.5f*u + onesixth) +
		    d * (onesixth*cubed));
};

// returns the first derivative of a cubic B-Spline
template <class T> T spline_bspline_firstder(T a, T b, T c, T d, float u) {
	const float threetwo = 3.0f / 2.0f;

	float omu = 1.0f - u;
	float omu_square = omu * omu;
	float omu_cube = omu_square *omu;
	float squared = u * u;
	float cubed = u * squared;
	return (a * (-0.5f * omu_square) +
		    b * (threetwo * squared - 2.0f*u) +
		    c * (-threetwo * squared + u + 0.5f) +
		    d * (0.5f * squared));
};

// Linear interpolation
template <class T> T  lerp(T a, T b, float u) {
	return T(a + ((b - a))*u);
};

// Cosine interpolation
template <class T> T cos_interp(T a, T b, float x ) {
	float ft = x * 3.1415927;
	float f = (1 - cos(ft)) * 0.5;
	return a*(1-f) + b*f;
};

// Sine^2 nonlinear smoothing
inline float nonlinear_smooth(float val) {
	float partial = sin(val * 1.57079632679f);
	return partial*partial;
}



// clamp func
template <class T> void clamp(T &value, const T min, const T max) {
	if (value < min)
		value = min;
	else if (value > max)
		value = max;
	return;
};

// swap
template <class T> void swap(T &a, T &b) {
	T temp(a);
	a = b;
	b = temp;
}

#endif