#pragma once

#include "common.h"

#include "globals.h"

class HField;

struct TriangleIndexEntry {
	int v[4];	// reserve space for 4 just in case we're using quads...
};

class TerrainMesh {
public:
	TerrainMesh();
	~TerrainMesh(); // Mesh does NOT release resources on destructor! Call DeleteMesh instead!

// Creation functions to generate a mesh from a Heightfield
	bool CreateRegularMesh(HField *hf, HField *mask, int create_flags = 0);
	bool CreateOptimizedMesh(HField *hf, HField *mask, int quality, int create_flags = 0);

	void CopyFrom(TerrainMesh &other);
	void DeleteMesh();

// Export function: generate a heightfield from a mesh
	HField *ExportHeightField(SizeData &hfsize);

// Vertex list
	Vector3 *vertex_data;
	int num_vertex;
// Face list
	TriangleIndexEntry *face_data;
	int num_face;

	Vector2 bx,by,bz;		// bounding planes on X,Y,Z

	int coord_system;		// world scaling
	int primitive_type;		// 0 = tris, 1 = quads
	bool use_tex_coords;	// if the mesh contains texture coordinates
	bool use_normals;		// if the mesh contains vertex normal data
};

class TerrainMeshImporter {
public:
	static bool InputOBJMesh(const char *filename, TerrainMesh &mesh);
};

class TerrainMeshExporter {
public:
	static bool OutputOBJMesh(const char *filename, TerrainMesh &mesh);
	static bool OutputLWOMesh(const char *filename, TerrainMesh &mesh);
};