// This can be used as a template for your own plugin DLL
#include "stdafx.h"
#include "DllExample.h"

// Include all of the headers for the devices I want to include in this DLL here
#include "Inverter.h"
#include "SimpleGenerator.h"
#include "SimpleOutput.h"
#include "SimpleWorldEffect.h"

// This function must be implemented in every plugin DLL. It sets the WM core library pointer to the
// global core provided by the WM Plugin host. Failure to implement this function as shown will cause
// wonky memory heap errors.
void SetupGlobalAccess(WMGlobalStruc *global) {
	WMGlobal = global;
};

// Each plugin must implement this function or WM will not load it. This ensures that the core
// the plugin is linking against is compatible with the core running.
int  GetHeaderVersion() {
	return WM_PLUGIN_HEADER_VERSION;
};

// Retrieve the information&pointers that the device world will use to create and destroy your devices.
// WM will retrieve devices starting at #0. If the function returns true, WM will proceed to the next number
// a return value of false indicates that no more devices are available. The simplest implementation is shown
// below...
bool GetDevPlugin(const int i, DevPluginStruc *result) {
	if (!result)
		return false;

	DevPluginStruc dat;	// Plugin device info
	switch ( i ) {
		case 0:			// Device 0: Inverter
			{
			Device *dev =  new Inverter;	// Instantiate the device temporarily to get at its name and life funcs
			dat.lifedata = dev->GetLifeVars();
			strcpy_s(dat.name, dev->GetTypeName());
			dat.type = TYPE_FILTER;
			delete dev;

			*result = dat;
			}
			break;
		case 1:			// Device 1: Simple Generator
			{
			Device *dev =  new SimpleGenerator;	// Instantiate the device temporarily to get at its name and life funcs
			dat.lifedata = dev->GetLifeVars();
			strcpy_s(dat.name, dev->GetTypeName());
			dat.type = TYPE_GENERATOR;
			delete dev;
			*result = dat;
			}
			break;
		case 2:			// Device 2: Simple Output
			{
			Device *dev =  new SimpleOutput;	// Instantiate the device temporarily to get at its name and life funcs
			dat.lifedata = dev->GetLifeVars();
			strcpy_s(dat.name, dev->GetTypeName());
			dat.type = TYPE_OUTPUT;
			delete dev;
			*result = dat;
			}
			break;
		case 3:			// Device 3: Simple World-scaling example
			{
			Device *dev =  new SimpleWorldEffect;	// Instantiate the device temporarily to get at its name and life funcs
			dat.lifedata = dev->GetLifeVars();
			strcpy_s(dat.name, dev->GetTypeName());
			dat.type = TYPE_GENERATOR;
			delete dev;
			*result = dat;
			}
			break;
		default:		// Out of devices; return false.
			return false;
	}
	return true;
};
