#include "StdAfx.h"
#include "simpleoutput.h"
#include "../core/HField.h"

Device *SimpleOutputMaker() { return new SimpleOutput; };
void SimpleOutputKiller(Device *spr) { delete spr; };


SimpleOutput::SimpleOutput(void)
{
	strcpy_s(filename, "null");
	lifeptrs.maker = SimpleOutputMaker;
	lifeptrs.killer = SimpleOutputKiller;
	strncpy(lifeptrs.nametag, "EXSO", 4);
	SetLinks(1, 0); // inheriting from Output assumes this, but we'll re-emphasize it.
}

SimpleOutput::~SimpleOutput(void)
{
}

// You are REQUIRED to write out and read in your 4-byte Device ID tag, as shown below. It is also recommended that your device
// have an internal version number that you keep track of to allow backwards-compatibility if you ever change your Device save layout...
// Also, make sure you call your super class's Load and Save functions as shown here. Failure to do so will cause major problems.
bool SimpleOutput::Load(std::istream &in) {
	char tag[5];
	in.read( tag, 4);
	if (strncmp(tag, lifeptrs.nametag, 4) == 0) {
		int ver = 0;
		in.read((char*) &ver, 1);
		return Output::Load(in);
	}
	else
		return false;
};

bool SimpleOutput::Save(std::ostream &out) {
	out.write( lifeptrs.nametag, 4);
	int ver = 1;
	out.write((char*) &ver, 1);
	return Output::Save(out);
};

// If ANY device has any data that is not stored by the parameter data system it must have a
// doClone function written to properly copy that data when the device is copied.
// In this case, we have the filename variable that needs to be copied.
void SimpleOutput::doClone(Device *other) {
	SimpleOutput *simp = (SimpleOutput*) other;
	strcpy_s(filename, simp->filename);
}

bool SimpleOutput::doOutput(OutputSet &outset, OUTPUT_MODE output_mode) {
	assert(false && "TODO: doOutput() implementation!");
	// TODO: DO YOUR OUTPUT!
	return false;
};



bool SimpleOutput::Activate(BuildContext &context) {
	HField *output = NULL;

	output = HF(RetrieveData(0, context));
	if (!output)
		return false;

	// OutputSets are a way that WM manages terminal heightfields (heightfields at the end of the network that should be kept around)
	// Use the constructor version like this:
	OutputSet output_set(output);
	// 	or the empty constructor and set(input_port, PACKET) version like this:
	// OutputSet output_set;
	// output_set.set(0, output);


// Only perform the Saving action if we're NOT activating the device to build the preview.
// (ie, we're doing a final build)
	if (context.pstate != P_PREVIEW) {
		// Save output to disk
		doOutput(output_set, OUTPUT_NORMAL);
	}

// Output devices have no output ports. Because of this, they normally would have no way of keeping
// their contents for WM to display. In addition, it is nice to keep internal track of all terminal
// (output) heightfields for other purposes. A call to StoreOutputContents performs this; a special
// function defined in the Output class, it keeps the output heightfield for later use. You don't need to worry about the details;
// just pass your output set to the StoreOutputContents functions at the end of your Activate() call. 
	StoreOutputContents(output_set, NULL, context);
	return true;
};



