#include "StdAfx.h"
#include "simpleworldeffect.h"
#include "..\core\HField.h"

Device *SimpleWorldEffectMaker() { return new SimpleWorldEffect; };
void SimpleWorldEffectKiller(Device *spr) { delete spr; };

SimpleWorldEffect::SimpleWorldEffect(void)
{
	lifeptrs.maker = SimpleWorldEffectMaker;
	lifeptrs.killer = SimpleWorldEffectKiller;
	strncpy(lifeptrs.nametag, "EXSW", 4);
	SetLinks(0, 1);	// generators have 0 inputs by default. we'll just re-emphasize that..

	AddParam(Parameter("Spacing", 1.0f, 0.1f, 10.0f));
}

#define SWE_SPACING		ParmFRef("Spacing")

SimpleWorldEffect::~SimpleWorldEffect(void)
{
}

// You are REQUIRED to write out and read in your 4-byte Device ID tag, as shown below. It is also recommended that your device
// have an internal version number that you keep track of to allow backwards-compatibility if you ever change your Device save layout...
// Also, make sure you call your super class's Load and Save functions as shown here. Failure to do so will cause major problems.
bool SimpleWorldEffect::Load(std::istream &in) {
	char tag[5];
	in.read( tag, 4);
	if (strncmp(tag, lifeptrs.nametag, 4) == 0) {
		int ver = 0;
		in.read((char*) &ver, 1);
		return Generator::Load(in);
	}
	else
		return false;
};

bool SimpleWorldEffect::Save(std::ostream &out) {
	out.write( lifeptrs.nametag, 4);
	int ver = 1;
	out.write((char*) &ver, 1);
	return Generator::Save(out);
};

bool SimpleWorldEffect::Activate(BuildContext &context) {
// Create a new heightfield using current specified world size
	HField *map = GetNewHF(context.GetWorldSize());

	float spacing = SWE_SPACING;

// get the current origin, modified by the scaling mode and any coordinate offset 
//   supplied on the coordinate input to the device. The origin is in WORLD SPACE.
	CoordF origin = getOrigin();	

// Iterate over the entire heightfield, supplying a value at each location.
// We're going to make a sine-type pattern based on the distance to the origin.
// We want it to produce the correct result no matter the aspect ratio, scaling, or panning of the heightfield.
// The easiest method to do so is to use the function pair
//	CoordF TransformCoord_HFToWorld(SizeData &sd, CoordF &device_loc);
//	CoordF TransformCoord_WorldToHF(SizeData &sd, CoordF &world_loc);
// as they handle all coordinate transforms to and from heightfield and world space for you.

	SizeData sizedata = map->getSizeData();

	for (int y = 0; y < map->h(); y++) {
		for (int x = 0; x < map->w(); x++) {
			// find a world-space coordinate for this heightfield coordinate
			CoordF world_loc = TransformCoord_HFToWorld(context, CoordF(float(x),float(y)) );

			CoordF delta = world_loc - origin;
			// distance in worldspace from this heixel to the origin
			float distance = delta.length();

// do the cosine pattern, and rescale the result from -1..1 to 0..1.
			float result_value = (0.5f * cos(3.14159f * distance * spacing)) + 0.5f;
			(*map)[Coord(x,y)] = result_value;
		}
		context.ReportProgress(this, y, map->h());
	}


// Handoff to output(s)
	StoreData(map,0, context); // pass the heightfield to our output stage.
// Success!
	return true;
}
